/*
nom du fichier	: NetquizScormWrapper.js
version			: 1.00
auteur			: alain trottier
creation		: 2007/02/02
modification	: 2007/03/02

definition		: Interface entre Netquiz et les LMS supportant l'environnement d'excution de SCORM 1.2.
*/

//====================================================================================================================
//================= CONSTRUCTEUR  NetquizScormWrapper_V ==============================================================
function NetquizScormWrapper_V () {

	this.scormAPI_O				= null	// Rfrence de l'objet API fournir par le LMS, initialiser par la mthode scormWrapperConnect_B

	// Public, Interface du wrapper scorm -- API
	this.setLanguage_V			=	scormWrapperSetLanguage_V
	this.getStudentName_S		=	scormWrapperGetStudentName_S
	this.setSuspendData_V		= 	scormWrapperSetSuspendData_V
	this.getSuspendData_S		=	scormWrapperGetSuspendData_S
	this.getScore_S				=	scormWrapperGetScore_S
	this.setScorePond_V 		=	scormWrapperSetScorePond_V
	this.getScorePond_S 		=	scormWrapperGetScorePond_S
	this.setQuestionScore_V		= 	scormWrapperSetQuestionScore_V
	this.getQuestionScore_S		= 	scormWrapperGetQuestionScore_S
	this.getQuestionPond_S	 	= 	scormWrapperGetQuestionPond_S		// Utile pour afficher la pondration dans le Quiz
	this.finish_V				=	scormWrapperFinish_V				// Termine la communication entre le SCO et LMS.
	
	// Mthodes priv, appeler seulement par les mthode de cette objet.
	this.connect_B 				=	scormWrapperConnect_B
	this.find_O					= 	scormWrapperFind_O	
	this.lmsInitialize_B		= 	scormWrapperLmsInitialize_B
	this.verifyLastLmsCall_B	=	scormWrapperVerifyLastLmsCall_B
	this.commit_B				=	scormWrapperCommit_B				// 
	this.setScore_V				= 	scormWrapperSetScore_V 				
	this.updateScore_V			= 	scormWrapperUpdateScore_V 			// Utiliser par setQuestionScore_V pour cumul le total du pointage de l'usager du quiz.
	this.setQuestionPond_V 		= 	scormWrapperSetQuestionPond_V

	// Mthode priv lier au teste de conformiter du model cmi, voir aussi this.cfg_O.popUpConformanceCmi_B
	this.testOfConformanceCmi_V	=	scormWrapperTestOfConformanceCmi_V	
 	this.getValueFromCmiObj_S	=	scormWrapperGetValueFromCmiObj_S
 	this.setValueToCmiObj_V		=	scormWrapperSetValueToCmiObj_V
	this.callLmsWithCmi_V		=	scormWrapperCallLmsWithCmi_V

	// C'est ici et seulement ici qu'on configuration de l'objet
	this.cfg_O = {
		defaultLanguage_S		: "fr",		// Langue par defaut des messages gentil pour l'usager de Netquiz.
		showAlertError_B		: false,		// Gentil messages pour l'usager de Netquiz.
		debugMode_B				: false,		// METTRE CETTE CONDITION  FALSE LORS DE LA VERSION FINAL.
		traceMode_B				: false, 	// METTRE CETTE CONDITION  FALSE LORS DE LA VERSION FINAL. Type de messages obstruant, encombrant.
		popUpConformanceCmi_B	: false, 	// METTRE CETTE CONDITION  FALSE LORS DE LA VERSION FINAL. Pour que le popup fonctionne faut dactiver les block popup du fureteur.
		nOccurence_I			: 10,		// Nombre maximun de fentre parent ou de cadre qu'on doit remonter pour rechercher l'API du scorm.
		pondMin_S				: "0",		// Valeur par defaut de la pondration minimum.
		pondMax_S				: "100"		// Valeur par defaut de la pondration maximun.
		}

	// Il est important de comprendre que l'objet CMI joue deux roles! D'abord il nous permet de dfinir des testes de conformit au
	// model des donnes CMI puis suite  ces testes et selon les rsultats il nous permet de vrifier si ces lments du CMI sont supporter par le LMS.
	// Voici comment, this.cmi = {suspend_data : "Set,NetQuiz,Get"} signifie que nous voulons tester
	// l'appel this.scormAPI_O.LMSSetValue ('cmi.suspend_data','NetQuiz') et l'appel this.scormAPI_O.LMSGetValue ('cmi.suspend_data')
	// puis selon les rsultats de ces appels, les chaines dfinissant les testes (ex: "Set,NetQuiz,Get") sont remplac par des valeurs
	// boolien. Ainsi on pourra tester simplement si les lments du model CMI sont supporter, exemple if (this.cmi.suspend_data).
	// Noter que nous profitons de ces testes pour initialiser la pondration du score.

	this.cmi = 	{
		core :	{
				student_name	: "Get",
				score			: 	{
									min		: "Set,0,Get",
									max		: "Set,100,Get"
									}
				},
		objectives : {
				_children		: "Get",
				_count			: "Get"
				},
		suspend_data 			: "Set,NetQuiz,Get" 
		}

	// Dclarations des variables de la classe
	this.states_O				=	{
									lmsInitialize_B : 	false,
									language_S      : 	"",
									msg_O			: 	null,
									tableHTML_S     : 	"",
									objectives_O	:	{} // Utile lorsque le LMS ne supporte pas cmi.objectives
									}

	// Dfinition de tous les messages gentil pour l'usager de NetQuiz (associer  showAlertError_B).
	this.intl18n_O				= 	{
									fr_O 	: 	{
										unableFindAdapterAPI_S		: "Erreur, ne peut trouver l'adapteur d'API",
										failToInitialiseLMS_S		: "Echecs de l'initialisation du LMS",
										lmsNotInialise_S			: "Erreur LMS non initialiser",
										// failToCommit_S			: "Echecs de la fonction LMSCommit",
										failToGetStudiantName_S		: "Echecs ne peut lire le nom de l'etudiant",
										failToSetSuspendData_S		: "Echecs ne peut enregistrer les donnees suspendu",
										failToGetSuspendData_S		: "Echecs ne peut lire les donnees suspendu",
										failToSetScore_S			: "Echecs ne peut enregistrer la note de l'etudiant",
										failToGetScore_S			: "Echecs ne peut lire la note de l'etudiant",
										failToSetPond_S				: "Echecs ne peut enregistrer la ponderation",
										failToGetPond_S				: "Echecs ne peut lire la ponderation",
										failToUpdateScore_S			: "Echecs ne peut mettre a jour le pointage total",
										failToSetQuestionScore_S	: "Echecs ne peut enregistrer la note de l'objectif de l'etudiant",
										// failToGetQuestionScore_S	: "Erreur: Echecs ne peut lire la note de l'objectif de l'etudiant",
										failToSetQuestionPond_S		: "Echecs ne peut enregistrer la ponderation de la question",
										failToGetQuestionPond_S		: "Echecs ne peut lire la ponderation de la question",
										failToFinish_S				: "Echecs de la fonction LMSFinish"
										},
									en_O	: 	{
										unableFindAdapterAPI_S		: "Error, unable to find an API adapter",
										failToInitialiseLMS_S		: "Error, fail to initialie LMS",
										lmsNotInialise_S			: "Error LMS not initialiser",
										// failToCommit_S			: "Fail to LMSCommit",
										failToGetStudiantName_S		: "Fail to get the studiant name",
										failToSetSuspendData_S		: "Fail to set suspend data",
										failToGetSuspendData_S		: "Fail to get suspend data",
										failToSetScore_S			: "Fail to set the studiant score",
										failToGetScore_S			: "Fail to get the studiant score",
										failToSetPond_S				: "Fail to set the the ponderation",
										failToGetPond_S				: "Fail to get the the ponderation",
										failToUpdateScore_S			: "Fail to update the total score total",
										failToSetQuestionScore_S	: "Fail to set the objective studiant score",
										// failToGetQuestionScore_S	: "Fail to get the objective studiant score",
										failToSetQuestionPond_S		: "Fail to set ponderation of the question",
										failToGetQuestionPond_S		: "Fail to get ponderation of the question",
										failToFinish_S				: "Fail to LMSFinish"
										}
									}
	
	// ================ Let GO! ===============
	this.setLanguage_V ()
	this.connect_B ()
	this.testOfConformanceCmi_V ()
	// ========================================
	}

// ---------------------------------------------------------------------------------------------------
// Mthode connect_B
function scormWrapperConnect_B () {

	// recherche dans les cadres parent
	if ((window.parent) && (window.parent != window)){
	   	this.scormAPI_O = this.find_O (window.parent)
  		}
	
	// recherche dans les fenetres parent
	if ((this.scormAPI_O == null) && (window.opener != null)){
		this.scormAPI_O = this.find_O (window.opener)
  		}

	if (this.scormAPI_O != null) {
		if (this.lmsInitialize_B ()) return true
		else return false
		}
	else {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.unableFindAdapterAPI_S)
		}
	return false
	}

// ----------------------------------------------------------------------------------------	
// Mthode find_O
function scormWrapperFind_O (win_O) {
	var nTime_I = 0
	while ((win_O.API == null) && (win_O.parent != null) && (win_O.parent != win_O)) {
		nTime_I ++;
    	if (nTime_I > this.cfg_O.nOccurence_I) return null
    	win_O = win_O.parent
  		}
  	return win_O.API
	}

// ----------------------------------------------------------------------------------------	
// Mthode lmsInitialize_B
function scormWrapperLmsInitialize_B () {
	var result_S = this.scormAPI_O.LMSInitialize ("")
	if (result_S == "true") {
		this.states_O.lmsInitialize_B = true
		return true
		}
	else {
		this.states_O.lmsInitialize_B = false
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToInitialiseLMS_S)
		return false
		}
	}

// ---------------------------------------------------------------------------------------------------
// Mthode setLanguage_V
function scormWrapperSetLanguage_V (lang_S) {
	lang_S = (arguments.length == 1) ? lang_S : this.cfg_O.defaultLanguage_S;
	switch (lang_S) {
		case "en"	:	this.msg_O					= this.intl18n_O.en_O;
						this.states_O.language_S	= "en";
						break;

		case "fr"	:	;
		default		: 	this.msg_O					= this.intl18n_O.fr_O;
						this.states_O.language_S	= "fr"
		}
	}

// ----------------------------------------------------------------------------------------	
// Mthode verifyLastLmsCall_B
function scormWrapperVerifyLastLmsCall_B () {
	if (!this.states_O.lmsInitialize_B) return
		
	var lastErrorCode_S = this.scormAPI_O.LMSGetLastError ()
	if (this.cfg_O.debugMode_B) {
		switch (lastErrorCode_S) {
			case "0"	:	// if (this.cfg_O.traceMode_B) alert ("Code de retour du LMSGetLastError indique aucune erreur"); 
							break;
			case "101" 	:	alert ("LMSGetLastError: 101 General exception") ; break;
			case "201" 	:	alert ("LMSGetLastError: 201 Invalid argument error") ; break;
			case "202" 	:	alert ("LMSGetLastError: 202 Element cannot have children") ; break;
			case "203" 	:	alert ("LMSGetLastError: 203 Element not an array - cannot have count") ; break;
			case "301" 	:	alert ("LMSGetLastError: 301 Not initialized") ; break;
			case "401" 	:	alert ("LMSGetLastError: 401 Not implemented error") ; break;
			case "402" 	:	alert ("LMSGetLastError: 402 Invalid set value, element is a keyword") ; break;
			case "403" 	:	alert ("LMSGetLastError: 403 Element is read only") ; break;
			case "404" 	:	alert ("LMSGetLastError: 404 Element is write only") ; break;
			case "405" 	:	alert ("LMSGetLastError: 405 Incorrect Data Type") ; break;
			default  	:	alert ("unknow return code of LMSGetLastError");
			}
		}

	if (lastErrorCode_S == "0") return true
	else return false
	}

// ---------------------------------------------------------------------------------------------------
// Mthode getStudentName_S
function scormWrapperGetStudentName_S () {
	if (!this.states_O.lmsInitialize_B) return ""
	
	var studiantName_S = this.scormAPI_O.LMSGetValue ('cmi.core.student_name')
	if (this.verifyLastLmsCall_B ()) return studiantName_S
	if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToGetStudiantName_S)
	
	return ""
	}

// ---------------------------------------------------------------------------------------------------
// Mthode setSuspendData_V
function scormWrapperSetSuspendData_V (suspendData_S) {
	if (!this.states_O.lmsInitialize_B) return

	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.suspend_data) return

	this.scormAPI_O.LMSSetValue ('cmi.suspend_data',suspendData_S)
	if (this.verifyLastLmsCall_B () && this.commit_B ()) return
	if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetSuspendData_S)
	}

// ---------------------------------------------------------------------------------------------------
// Mthode getSuspendData_V
function scormWrapperGetSuspendData_S () {
	if (!this.states_O.lmsInitialize_B) return ""
	
	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.suspend_data) return

	var suspendData_S = this.scormAPI_O.LMSGetValue ('cmi.suspend_data')
	if (this.verifyLastLmsCall_B ()) return suspendData_S
	if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToGetSuspendData_S)

	return ""
	}

// ---------------------------------------------------------------------------------------------------
// Mthode setScore_V
function scormWrapperSetScore_V (score_S) {
	if (!this.states_O.lmsInitialize_B) return
	
	this.scormAPI_O.LMSSetValue ('cmi.core.score.raw',score_S)
	if (this.verifyLastLmsCall_B () && this.commit_B ()) return
	if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetScore_S)
	}

// ---------------------------------------------------------------------------------------------------
// Mthode getScore_S
function scormWrapperGetScore_S () {
	if (!this.states_O.lmsInitialize_B) return ""

	var score_S = this.scormAPI_O.LMSGetValue ('cmi.core.score.raw')
	if (this.verifyLastLmsCall_B ()) return score_S
	if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToGetScore_S)

	return ""
	}

// ---------------------------------------------------------------------------------------------------
// Mthode updateScore_V
function scormWrapperUpdateScore_V () {
	if (!this.states_O.lmsInitialize_B) return
	var totalScore_I = 0

	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.objectives._count) {
		for (p_O in this.states_O.objectives_O) totalScore_I += parseInt (this.states_O.objectives_O [p_O].raw_S)
		this.setScore_V (totalScore_I.toString())
		return
		}

	var count_S = this.scormAPI_O.LMSGetValue ('cmi.objectives._count')
	if (!this.verifyLastLmsCall_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToUpdateScore_S)
		return		
		}

	var count_I = (count_S == "") ? 0 : parseInt (count_S)
	
	
	// Si questionID_S existe on remplace la note par la valeur de score_S, sinon on ajoute la note au bout vecteur.
	for (var i=0; i < count_I; i++) {
		var n_S = i.toString()		
		var raw_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".score.raw")
		if (!this.verifyLastLmsCall_B ()) {
			if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToUpdateScore_S)
			return
			}
		totalScore_I += parseInt (raw_S)
		}

	this.setScore_V (totalScore_I.toString())
	}

// ---------------------------------------------------------------------------------------------------
// Mthode setScorePond_V -- Set la pondration du quiz
// Le parametres pondMax peut tre de type des string ou de type integer, cette mthode s'accomode avec les deux.
function scormWrapperSetScorePond_V (pondMax) {
	if (!this.states_O.lmsInitialize_B) return

	if (arguments.length != 1) {
		if (this.cfg_O.debugMode_B) alert ("Erreur setScorePond_V: parametre manquant")
		return;
		}
	else {
		var pondMax_I = (typeof (pondMax) ==  "integer")
			? pondMax
			: (isNaN (parseInt (pondMax)))
				? 0
				: parseInt (pondMax);

		var pondMax_S = pondMax_I.toString()
		}	

	pondMax_S =	(100 < pondMax_I || 0 > pondMax_I) ? this.states_O.pondMax_S : pondMax_S;

	this.scormAPI_O.LMSSetValue ('cmi.core.score.max',pondMax_S)

	if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetPond_S)
		return
		}
		
	// on initialise la valeur minimum de la ponderation  par la valeur par defaut
	this.scormAPI_O.LMSSetValue ('cmi.core.score.min',this.cfg_O.pondMin_S)

	if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetPond_S)
		return
		}
	}

// ---------------------------------------------------------------------------------------------------
// Mthode getScorePond_S
function scormWrapperGetScorePond_S () {
	if (!this.states_O.lmsInitialize_B) return ""
	
	var pond_S = this.scormAPI_O.LMSGetValue ('cmi.core.score.max')
	if (!this.verifyLastLmsCall_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToGetPond_S)
		return ""
		}

	return pond_S;
	}

// ---------------------------------------------------------------------------------------------------
// Mthode setQuestionScore_V
// Cette mthode est populaire alors les validations sont de vigueur.
// Les parametres score et maxPond peuvent tre de type des string ou de type integer, cette mthode s'accorde avec les deux.
function scormWrapperSetQuestionScore_V (questionID_S, score, maxPond) {
	if (!this.states_O.lmsInitialize_B) return;
	
	if (arguments.length != 3) { // Un oublie?
		if (this.cfg_O.debugMode_B) alert ("Erreur setQuestionScore_V: parametre manquant")
		return;
		}
	else {
		var score_I = (typeof (score) ==  "integer")
			? score
			: (isNaN (parseInt (score)))
				? 0
				: parseInt (score);

		var score_S = score_I.toString()
		
		var maxPond_I = (typeof (maxPond) ==  "integer")
			? maxPond
			: (isNaN (parseInt (maxPond)))
				? 0
				: parseInt (maxPond);

		var maxPond_S = maxPond_I.toString()
		
		if (maxPond_I < score_I) { // Trop fort
			if (this.cfg_O.debugMode_B) alert ("Erreur setQuestionScore_V: La note est superieur a la ponderation")
			return;
			}
		}

	// Initalise la ponderation
	this.setQuestionPond_V (questionID_S, maxPond_S)

	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.objectives._count) {
		// La question avec cette ID doit ncessairement exister puisque nous venons juste de setter la pondration pour cette question
		if (this.states_O.objectives_O[questionID_S]) {
			this.states_O.objectives_O[questionID_S].raw_S = score_S
			this.updateScore_V () // UpdateScore_V, ce dernier vas s'occuper de mettre  jour le pointage cumulatif.
			}
		return
		}

	// Enfin on vas s'occuper d'initaliser le score de la question.
	var count_S = this.scormAPI_O.LMSGetValue ('cmi.objectives._count')
	if (!this.verifyLastLmsCall_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionScore_S)
		return
		}

	var count_I = (count_S == "") ? 0 : parseInt (count_S);
	count_S = (count_S == "") ? 0 : count_S;
	var result_S,id_S,n_S = ""

	// Si questionID_S existe on remplace la note par la valeur de score_S, sinon on ajoute la note au bout vecteur.
	for (var i=0; i < count_I; i++) {
		n_S = i.toString()		
		id_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".id")
		if (!this.verifyLastLmsCall_B ()) {
			if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionScore_S)
			return
			}
		 
		if (id_S == questionID_S) {
			this.scormAPI_O.LMSSetValue ("cmi.objectives." + n_S + ".score.raw",score_S)
			if (!this.verifyLastLmsCall_B () || !this.commit_B()) {
				if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionScore_S)
				return
				}
			this.updateScore_V () 	// UpdateScore_V, ce dernier vas s'occuper de mettre  jour le pointage cumulatif.
			return
			}
		}

	// Si nous en sommes ici c'est qu'on set cette note pour la premier fois
	this.scormAPI_O.LMSSetValue ("cmi.objectives." + count_S + ".id",questionID_S)
	if (!this.verifyLastLmsCall_B () || !this.commit_B()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionScore_S)
		return
		}

	this.scormAPI_O.LMSSetValue ("cmi.objectives." + count_S + ".score.raw",score_S)
	if (!this.verifyLastLmsCall_B () || !this.commit_B()) {
		if (cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionScore_S)
		return
		}
	
	this.updateScore_V () // UpdateScore_V, ce dernier vas s'occuper de mettre  jour le pointage cumulatif.
	}

// ---------------------------------------------------------------------------------------------------
// Mthode getQuestionScore_S
function scormWrapperGetQuestionScore_S (questionID_S) {
	if (!this.states_O.lmsInitialize_B) return ""

	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.objectives._count) {
		if (this.states_O.objectives_O[questionID_S]) {
			return this.states_O.objectives_O[questionID_S].raw_S
			}
		return ""
		}

	var count_S = this.scormAPI_O.LMSGetValue ('cmi.objectives._count')
	this.verifyLastLmsCall_B ()
	var count_I = parseInt (count_S)
	var id_S,n_S = ""

	// Recherche de questionID_S.
	for (var i=0; i < count_I; i++) {
		n_S = i.toString()		
		id_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".id")
		if (!this.verifyLastLmsCall_B ()) break;
		if (id_S == questionID_S) {
			var raw_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".score.raw")
			if (this.verifyLastLmsCall_B ()) return raw_S
			else break;
			}
		}

	if (this.cfg_O.debugMode_B) alert ("Erreur getQuestionScore_S: La question avec l'id [" + questionID_S + "] est introuvable")
	return ""
	}

// ---------------------------------------------------------------------------------------------------
// Mthode setQuestionPond_V
// Note: on intialise aussie la pondration minimum avec la valeur par defaut
function scormWrapperSetQuestionPond_V (questionID_S, pondMax_S) {
	if (!this.states_O.lmsInitialize_B) return
	
	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.objectives._count) {
		// SI la question avec cette ID existe
		if (this.states_O.objectives_O[questionID_S]) {
			this.states_O.objectives_O[questionID_S].min_S = "0"
			this.states_O.objectives_O[questionID_S].max_S = pondMax_S
			}
		else {
			this.states_O.objectives_O[questionID_S] = {min_S:"0",max_S:pondMax_S}
			}
		return
		}

	var count_S = this.scormAPI_O.LMSGetValue ('cmi.objectives._count')
	if (!this.verifyLastLmsCall_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
		return
		}

	var count_I = (count_S == "") ? 0 : parseInt (count_S);
	count_S = (count_S == "") ? 0 : count_S;
	var result_S,id_S,n_S = ""

	// Si questionID_S existe on remplace la ponderation par la valeur de pondMax_S, sinon on ajoute la ponderation au bout vecteur.
	for (var i=0; i < count_I; i++) {
		n_S = i.toString()		
		id_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".id")
		if (!this.verifyLastLmsCall_B ()) {
			if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
			return
			}
			
		if (id_S == questionID_S) {
			this.scormAPI_O.LMSSetValue ("cmi.objectives." + n_S + ".score.max",pondMax_S)
			
			if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
				if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
				return
				}
			
			// On s'empresse de d'initaliser la valeur minimum de la pondration puisque nous y somme.
			this.scormAPI_O.LMSSetValue ("cmi.objectives." + n_S + ".score.min",this.cfg_O.pondMin_S)
			if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
				if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
				}
			return
			}
		}

	// Si questionID_S existe on remplace la note par la valeur de score_S, sinon on ajoute la note au bout vecteur.
	this.scormAPI_O.LMSSetValue ("cmi.objectives." + count_S + ".id",questionID_S)

	if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
		return
		}

	this.scormAPI_O.LMSSetValue ("cmi.objectives." + count_S + ".score.max",pondMax_S)
	if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
		return
		}

	// Au detour ont s'empresse de d'initaliser la pondration minimum
	this.scormAPI_O.LMSSetValue ("cmi.objectives." + count_S + ".score.min",this.cfg_O.pondMin_S)
	if (!this.verifyLastLmsCall_B () || !this.commit_B ()) {
		if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToSetQuestionPond_S)
		}
	}
	
// ---------------------------------------------------------------------------------------------------
// Mthode getQuestionPond_S
function scormWrapperGetQuestionPond_S (questionID_S) {
	if (!this.states_O.lmsInitialize_B) return ""

	// Dans le cas ou le LMS ne supporte pas cmi.objectives comme pour la platforme Claroline
	if (!this.cmi.objectives._count) {
		// SI la question avec cette ID existe
		if (this.states_O.objectives_O[questionID_S]) {
			return this.states_O.objectives_O[questionID_S].max_S
			}
		return ""
		}

	var count_S = this.scormAPI_O.LMSGetValue ('cmi.objectives._count')
	this.verifyLastLmsCall_B ()
	var count_I = parseInt (count_S)
	var id_S,n_S = ""

	// Recherche de questionID_S.
	for (var i=0; i < count_I; i++) {
		n_S = i.toString()		
		id_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".id")
		if (!this.verifyLastLmsCall_B ()) break;
		
		if (id_S == questionID_S) {
			var max_S = this.scormAPI_O.LMSGetValue ("cmi.objectives." + n_S + ".score.max")
			if (this.verifyLastLmsCall_B ()) return max_S
			else break;
			}
		}

	if (this.cfg_O.showAlertError_B) alert (this.msg_O.failToGetQuestionPond_S)
	return ""
	}

// ---------------------------------------------------------------------------------------------------
// Mthode commit_B
// Aucun message est gnrer la fonction appellante s'en occupe.
function scormWrapperCommit_B () {
	if (!this.states_O.lmsInitialize_B) return false
	
	var result_S = this.scormAPI_O.LMSCommit ('')
	if (result_S == "true") return true
	else return false
	}

// ---------------------------------------------------------------------------------------------------
// Mthode finish_V
function scormWrapperFinish_V () {
	if (!this.states_O.lmsInitialize_B) return

	var result_S = this.scormAPI_O.LMSFinish ('')
	if (result_S == "false" && this.cfg_O.showAlertError_B) alert (this.msg_O.failToFinish_S)
	else this.states_O.lmsInitialize_B = false // Fait en sorte qu'aucune appel drnavant aux mthode ne fonctionne.
	}
	
// ===================================================================================================
// Mthode lier au teste de conformit au model de donnes CMI
// ===================================================================================================
// ---------------------------------------------------------------------------------------------------
// Mthode setValueToCmiObj_V
function scormWrapperSetValueToCmiObj_V (code_S, value) {
	var t_O = code_S.split (".")
	code_S = t_O[0]
	for (var i=1; i < t_O.length; i++) code_S += (!isNaN (Number (t_O[i]))) ? "[" + t_O[i] + "]" : "." + t_O[i];

	var bool_S = value ? "true" : "false";

	var t_S = "this." + code_S + "= " + bool_S;
	eval (t_S)
	}

// ---------------------------------------------------------------------------------------------------
// Mthode getValueFromCmiObj_S
function scormWrapperGetValueFromCmiObj_S (code_S) {
	// Convertion de notation de code_S: exemple cmi.objectives.0.id devient cmi.objectives[0].id
	// parceque faire un eval sur (cmi.objectives.0.id) gnre une erreur JS du au 0.
	var t_O = code_S.split (".")
	code_S = t_O[0]
	for (var i=1; i < t_O.length; i++) code_S += (!isNaN (Number (t_O[i]))) ? "[" + t_O[i] + "]" : "." + t_O[i];

	code_S = "this." + code_S
	return eval (code_S)
	}
			
// ---------------------------------------------------------------------------------------------------
// Mthode callLmsWithCmi_V
function scormWrapperCallLmsWithCmi_V (cmi_S) {
	var testFail_B = false
	var array_O = (this.getValueFromCmiObj_S(cmi_S)).split(",")
	
	if (this.cfg_O.popUpConformanceCmi_B) this.states_O.tableHTML_S += "<tr>"
	if (array_O [0] == "Get") {
		this.scormAPI_O.LMSGetValue (cmi_S)	
		var code_S = this.scormAPI_O.LMSGetLastError ()		
		if (code_S == "0" && this.cfg_O.popUpConformanceCmi_B) {
			this.states_O.tableHTML_S += "<td>" + cmi_S + "<\/td><td><span style='color:green'>ok<\/span><\/td><td>n/a<\/td>"
			}
		else {
			testFail_B = true
			if (this.cfg_O.popUpConformanceCmi_B) this.states_O.tableHTML_S += "<td>" + cmi_S + "<\/td><td><span style='color:red'>fail [" + code_S + "]<\/span><\/td><td>n/a<\/td>"
			}
		}
	if (array_O [0] == "Set") {
		this.scormAPI_O.LMSSetValue (cmi_S,array_O [1])
		var code_S = this.scormAPI_O.LMSGetLastError ()
		if (code_S == "0" && this.cfg_O.popUpConformanceCmi_B) this.states_O.tableHTML_S += "<td>" + cmi_S + "<\/td><td><span style='color:green'>ok<\/span><\/td>"
		else {
			testFail_B = true
			if (this.cfg_O.popUpConformanceCmi_B) this.states_O.tableHTML_S += "<td>" + cmi_S + "<\/td><td><span style='color:red'>fail [" + code_S + "]<\/span><\/td>"
			}
		this.scormAPI_O.LMSGetValue (cmi_S)
		var code_S = this.scormAPI_O.LMSGetLastError ()
		if (code_S == "0" && this.cfg_O.popUpConformanceCmi_B) {
			this.states_O.tableHTML_S += "<td><span style='color:green'>ok<\/span><\/td>"
			}
		else {
			testFail_B = true
			if (this.cfg_O.popUpConformanceCmi_B) this.states_O.tableHTML_S += "<td><span style='color:red'>fail [" + code_S + "] <\/span><\/td>"
			}
		}
	
	// Important on remplace les valeur des feuilles de l'objet this.cmi pas des valeurs booleen. C'est en testant sur ces dernires que le
	// programme sais si le modle de donne est implanter ou non. ex: if (this.cmi.objectives._count) 
	if (testFail_B) this.setValueToCmiObj_V (cmi_S, false)
	else this.setValueToCmiObj_V (cmi_S, true)

	if (this.cfg_O.popUpConformanceCmi_B) this.states_O.tableHTML_S += "<\/tr>"
	}

// ---------------------------------------------------------------------------------------------------
// Mthode testOfConformanceCmi_V
function scormWrapperTestOfConformanceCmi_V () {
	if (!this.states_O.lmsInitialize_B) return
	
	if (this.cfg_O.popUpConformanceCmi_B) {
		// Cration de l'entte du tableau du pop up
		this.states_O.tableHTML_S =	"<table style='margin:30px;font-size:11px;font-family:verdana;background-color:#d3d3d3' cellspacing='0' cellpadding='4' border='1'>" +
									"<tr style='font-weight:bold'><td>Model<\/td><td>Get<\/td><td>Set<\/td><\/tr>"
		}
		
	var this_O = this;
	// Fonction rcursive et anonyme qui traverse l'objet this.cmi.
	//  l'aide l'objet this.cmi la fonction construit le nom des lements du cmi ex: cmi.objectives._count,
	// puis  chaque feuille de l'objet this.cmi elle appel callLmsWithCmi_V.
	(function (o_O,p_S) {
		for (var o in o_O) {
			if (typeof o_O [o] != "object") { // feuille
				this_O.callLmsWithCmi_V (p_S + "." + o)
				continue
				}
			arguments.callee (o_O[o],p_S + "." + o)
			} 
		}) (this.cmi,"cmi")

	// Fini la construction du tableau et l'affiche dans un popUp.
	if (this.cfg_O.popUpConformanceCmi_B) {
		this.states_O.tableHTML_S += "<\/table>"
		var win_O = window.open('','CMI','width=360,height=400,scrollbars=yes')
		if (win_O) {
	    	win_O.focus();
    		win_O.document.body.innerHTML = this.states_O.tableHTML_S
			}
		else {
			if (this.cfg_O.debugMode_B) alert ("Verifier si le bloquage des fentres surgissantes est activer")
			}
		}
	}

